

proc Set_V767_Options {} {
    global Options V767_Base

    load_Options

   if {![info exists Options(v767.channels)]} {set Options(v767.channels) 4096}
   if {![info exists Options(v767.offset)]} {set Options(v767.offset) 0}
   if {![info exists Options(v767.shift)]} {set Options(v767.shift) 4}
   if {![info exists Options(v767_GroupBase)]} {set Options(v767_GroupBase) $V767_Base}
   set V767_Base   $Options(v767_GroupBase)

   if {![info exists Options(v767.acqmode)]} {set Options(v767.acqmode) 0}
   if {![info exists Options(v767.windowwidth)]} {set Options(v767.windowwidth) 300}
   if {![info exists Options(v767.windowoffset)]} {set Options(v767.windowoffset) -250}

    update_Options
}

proc Dump_Raw_VME_Data_V767 {} {
    global CAEN_ModuleIndex

   set info ""

   set maxmodule [inform vmecontrol inquire v767count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   for {set i 1} {$i <= $maxmodule} {incr i} {
      append info "\nV767#$i\n"
      execute-command 19 "Getting Raw Data for V767#$i" "1 $CAEN_ModuleIndex" 2
      set data [read-register EXEC.Response]

      incr CAEN_ModuleIndex
      
     foreach word $data {
        set word 0x$word
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 21 & 3]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 24 & 0x7f]
             set st [expr $word >> 23 & 1]
             set ed [expr $word >> 20 & 1]
             set tdc [expr $word  & 0xfffff]
             append info "[format "0x%8x    %8s   channel= %5i; start= %3i; edge= %3i; time= 0x%5x" $word $event_type $channel $st $ed $tdc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0x3ff]
             append info "[format "0x%8x    %8s   module= %5i; event number= %6i" $word $event_type $geo $counter]"
            }
            1 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set status [expr $word >> 24 & 7]
             set counter [expr $word & 0xffff]
             append info "[format "0x%8x    %8s   module= %5i; status=%2i; event data counter= %8i" $word $event_type $geo $status $counter]"
            }
            3 {
             append info "$word - fifo empty"
            }
         }
         append info "\n"
     }
   }
   return $info
}

proc define_V767_config {} {
    global VMEserver
    global V767_Base

   set maxmodule [inform vmecontrol inquire v767count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   if {$maxmodule > 0} then {DefineConfig "767 0 [format "0x%x" $V767_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v767#$i.Output_Word'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v767#$i.Output_Word"
          clear-footer
          error $m
      }
      DefineConfig "767 $i [string range [first $m] 2 end]"
   }

}

proc create_V767_spectra {} {
    global SpectrumServer V767_Base
    global Options

   set maxmodule [inform vmecontrol inquire v767count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}
   set maxadc 128

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module v767#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v767#$i.tdc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v767.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum v767#$i.tdc$j"
          clear-footer
          error $m
      } else {
         if {$j < 64} then {
             set group [expr $V767_Base + (($i - 1) * 2)]
             set item $j
         } else {
             set group [expr $V767_Base + (($i - 1) * 2) + 1]
             set item [expr $j - 64]
         }
          map_spectrum  "$spectrumname $group $item $Options(v767.offset) $Options(v767.shift) "
      }
    }
   }
}

proc do_V767_setup {} {
    global Options

   set maxmodule [inform vmecontrol inquire v767count]
   if {[string length $maxmodule] > 2} {set maxmodule 0}

   if {$maxmodule == 0} {return}
   
   starttask v767

   for {set i 1} {$i <= $maxmodule} {incr i} {

       inform v767 click-select-module $i
        
#    set window width
       set rq $Options(v767.windowwidth)
       for {set Tries 0} {$Tries < 10} {incr Tries 1} {
           inform v767 do_set_opcode 0x3000 $rq
           after 500
           set vl [inform v767 do_read_opcode 0x3100]
           if {$rq == $vl} {break}
           insert-log "Error setting V767#$i Window Width: requested $rq; set $vl"
       }

       update idletasks
       
#    set window offset
       set rq $Options(v767.windowoffset)
       for {set Tries 0} {$Tries < 10} {incr Tries 1} {
           inform v767 do_set_opcode 0x3200 $rq
           after 500
           set vl [inform v767 do_read_opcode 0x3300]
           if {$rq == $vl} {break}
           insert-log "Error setting V767#$i Window Offset: requested $rq; set $vl"
       }

       update idletasks
       
#    set acquisition mode 
       set rq $Options(v767.acqmode)
       for {set Tries 0} {$Tries < 10} {incr Tries 1} {
           inform v767 do_cmd_opcode 0x1[set rq]00
           after 500
           set vl [inform v767 do_read_opcode 0x1400]
           if {$rq == $vl} {break}
           insert-log "Error setting V767#$i Acquisition Mode: requested $rq; set $vl"
       }

#    data fifo flush - module clear does not work!!
       for {set Offset 0} {$Offset < 2048} {incr Offset 1} {
          set word [inform v767 do_read_register Output_Word]
          if {[expr $word >> 21 & 3] == 3} {break}    ;#  end of data 
       }
   }
}
